#!/usr/bin/python
# coding=utf-8


from bcc import BPF
from pyroute2 import IPRoute, IPDB
import sys


# Programma eBPF #
bpf_program = """\

	#include <bcc/proto.h>

	int filter (struct __sk_buff *skb) {
		struct ethernet_t *ether_header = 0;
		struct ip_t *ip_header = (struct ip_t *) sizeof (struct ethernet_t);
		struct udp_t *udp_header = (struct udp_t *) (sizeof (struct ethernet_t) + sizeof (struct ip_t));

		/* Filtra il Pacchetto se Non è IPv4 */
		if (ether_header->type != 0x0800)
			return -1;

		/* Lascia Passare il Pacchetto se è DNS */
		if (ip_header->nextp == 0x11 && (udp_header->dport == 0x0035 || udp_header->sport == 0x0035))
			return 0;

		/* Filtra il Pacchetto se Non è TCP */
		if (ip_header->nextp != 0x06)
			return -1;

		/* Lascia Passare il Pacchetto se è Diretto/Arriva verso/da l'Indirizzo IP scelto (130.192.181.193) */
		if (ip_header->src == 0x82c0b5c1 || ip_header->dst == 0x82c0b5c1)
			return 0;

		/* Filtra il Pacchetto */
		bpf_trace_printk ("Filtrato il Pacchetto: %lx->%lx\\n", ip_header->src, ip_header->dst);
		return -1;
	}

"""


# Inizializza IProute #
ipr = IPRoute()
ipr_db = IPDB (nl=ipr)
interface = ipr_db.interfaces.eth0

# Inizializza eBPF #
bpf = BPF (text=bpf_program)
pbr = bpf.load_func ("filter", BPF.SCHED_CLS)

# Inizializza TC #
ipr.tc ("add", "ingress", interface.index, "ffff:")
ipr.tc ("add-filter", "bpf", interface.index, ":1", fd=pbr.fd, name=pbr.name, parent="ffff:", action="drop", classid=1)

# Loop Principale #
print ("\n[*] Premere ctrl+c per Terminare...\n")
try:
	bpf.trace_print ()
finally:
	ipr.tc ("del", "ingress", interface.index, "ffff:")
	ipr_db.release ()
print ("\n")
