#!/usr/bin/python
# coding=utf-8


from bcc import BPF
import sys


# Programma eBPF #
bpf_prog = """\
	#define KBUILD_MODNAME "foo"
	#include <bcc/proto.h>
	#include <linux/ip.h>
	#include <linux/udp.h>

	int filter (struct xdp_md *ctx) {
		void* data = (void*)(long) ctx->data;
		void* data_end = (void*)(long) ctx->data_end;

		/* Inizializza gli Header */
		struct ethhdr *eth = data;
		uint64_t nh_off = sizeof (*eth);
		if (data + nh_off  > data_end)
			return XDP_PASS;
		struct iphdr *iph = data + nh_off;
		if ((void*) &iph[1] > data_end)
                       	return XDP_PASS;
		struct udphdr *udph = ((void *) iph) + sizeof (*iph);
		if ((void*) &udph[1] > data_end)
	              	return XDP_PASS;

		/* Lascia Passare il Pacchetto se Non è IPv4 */
		if (eth->h_proto != htons (ETH_P_IP))
			return XDP_PASS;

		/* Lascia Passare il Pacchetto se è DNS */
		if (iph->protocol == 0x11 && (udph->dest == 0x3500 || udph->source == 0x3500))
			return XDP_PASS;

		/* Filtra il Pacchetto se Non è TCP */
		if (iph->protocol != 0x06)
			return XDP_DROP;

		/* Lascia Passare il Pacchetto se è Diretto/Arriva verso/da l'Indirizzo IP scelto (130.192.181.193) */
		if (iph->saddr == 0xc1b5c082 || iph->daddr == 0xc1b5c082)
			return XDP_PASS;

		/* Filtra il Pacchetto */
		bpf_trace_printk ("Filtrato il Pacchetto: %lx->%lx\\n", iph->saddr, iph->daddr);
		return XDP_DROP;
	}
"""


# Inizializza eBPF & XDP #
bpf = BPF (text = bpf_prog)
bpf_func = bpf.load_func ("filter", BPF.XDP)
interface = "eth0"
bpf.attach_xdp (interface, bpf_func, 0)


# Loop Principale #
print ("\n[*] Premere ctrl+c per Terminare...\n")
try:
	bpf.trace_print ()
finally:
	bpf.remove_xdp (interface, 0)
print ("\n")
