/********************************
*				*
*	Protected Program	*
*				*
********************************/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <seccomp.h>

#define BUF_LEN 8
#define CMD_LEN 128



/* INIT_SECCOMP */
void init_seccomp () {
	scmp_filter_ctx seccomp_ctx;

	/* Inizializza la Libreria Seccomp (per la Creazione di una Whitelist) */
	if ((seccomp_ctx = seccomp_init (SCMP_ACT_KILL)) == NULL) {
		perror ("[ERRORE_init]");
		exit (0);
	}

	/* Definisci le Regole di Filtraggio */
	if (seccomp_rule_add (seccomp_ctx, SCMP_ACT_ALLOW, SCMP_SYS (fstat), 0) < 0) {
		perror ("[ERRORE_add_fstat]");
		exit (0);
	}
	if (seccomp_rule_add (seccomp_ctx, SCMP_ACT_ALLOW, SCMP_SYS (write), 0) < 0) {
		perror ("[ERRORE_add_write]");
		exit (0);
	}
	if (seccomp_rule_add (seccomp_ctx, SCMP_ACT_ALLOW, SCMP_SYS (read), 0) < 0) {
		perror ("[ERRORE_add_read]");
		exit (0);
	}
	if (seccomp_rule_add (seccomp_ctx, SCMP_ACT_ALLOW, SCMP_SYS (lseek), 0) < 0) {
		perror ("[ERRORE_add_lseek]");
		exit (0);
	}
	if (seccomp_rule_add (seccomp_ctx, SCMP_ACT_ALLOW, SCMP_SYS (exit_group), 0) < 0) {
		perror ("[ERRORE_add_exit]");
		exit (0);
	}

	/* Carica il Filtro Creato nel Kernel */
	if (seccomp_load (seccomp_ctx) < 0) {
		perror ("[ERRORE_load]");
		exit (0);
	}
}



/* MAIN */
int main () {
	char command [CMD_LEN];
	char buffer [BUF_LEN] = {'1', '2', '3', '4', '5', '6', '7', '8'};
	int el_num, i;

	/* Definisci le Regole di Filtraggio con Seccomp */
	init_seccomp ();

	/* Loop sul Comando da Eseguire */
	while (1) {
		printf ("Inserisci il Comando da Eseguire (read | write | exit): ");
		fflush (stdout);
		scanf ("%s", command);

		if (strcmp (command, "read") == 0) {	// Leggi Dati
			printf ("Inserisci il Numero di Elementi da Leggere: ");
			fflush (stdout);
			scanf ("%d", &el_num);
			if (el_num < 0 || el_num > BUF_LEN) {
				printf ("Il Numero di Elementi deve Essere Compreso tra 0 e 8!\n");
				fflush (stdout);
				continue;
			}
			for (i = 0; i < el_num; i++)
				printf ("0x%x ", buffer [i]);
			printf ("\n");
			fflush (stdout);

		} else if (strcmp (command, "write") == 0) {     // Scrivi Dati
			printf ("Inserisci il Numero di Elementi da Scrivere: ");
			fflush (stdout);
			scanf ("%d", &el_num);
			getchar ();
			if (el_num < 0 || el_num > BUF_LEN) {
				printf ("Il Numero di Elementi deve Essere Compreso tra 0 e 8!\n");
				fflush (stdout);
				continue;
			}
			for (i = 0; i < el_num; i++) {
				printf ("Inserisci l'Elemento n° %d: ", i + 1);
				fflush (stdout);
				scanf ("%c", &buffer [i]);
				getchar ();
			}
			printf ("\n");
			fflush (stdout);

		} else if (strcmp (command, "exit") == 0)	// Esci
			break;

		else {	// Errore
			printf ("Il Comando Inserito è Errato!\n");
			printf (command);
			printf ("\n");
			fflush (stdout);
		}
	}

	return 1;
}
