#!/bin/bash

###################
#		  #
# Network Manager #
#		  #
###################


# Controlla i Permessi di Root #
if [ "$EUID" -ne 0 ]; then
	echo -e "\n[!] Questo programma deve essere eseguito con permessi di root!\n"
	exit
fi


# Controlla i Parametri della Riga di Comando #
echo -e "\n\t\t--- Network Manager ---\n"
if [ $# -lt 1 ]; then
	echo -e "Utilizzo:\t$0 {COMANDO} [PARAMETRI_COMANDO]"
	echo -e "Per:\t\teseguire/distruggere/gestire una rete di container e/o un firewall\n"
	echo -e "\t{COMANDO}:"
	echo -e "\t\texec\t\tesegui comandi generici su una rete di container precedentemente creata"
	echo -e "\t\tinstall\t\tinstalla un firewall su una rete di container precedentemente creata"
	echo -e "\t\tstart\t\tesegui una rete di container e installa su essa un firewall"
	echo -e "\t\tstop\t\tdistruggi una rete di container e il firewall installato su essa"
	echo -e "\t\tuninstall\tdisinstalla un firewall precedentemente installato su una rete di container\n"
	exit
elif [ "$1" = "exec" ] && [ $# -lt 2 ]; then
	echo -e "Utilizzo:\t$0 exec {file_puppet_generico}"
	echo -e "Per:\t\teseguire comandi generici su una rete di container precedentemente creata\n"
	exit
elif [ "$1" = "install" ] && [ $# -lt 3 ]; then
	echo -e "Utilizzo:\t$0 install {file_puppet_init} {ID_FIREWALL}"
	echo -e "Per:\t\tinstallare un firewall su una rete di container precedentemente creata\n"
	echo -e "\t{ID_FIREWALL}:"
	echo -e "\t\t1\tebtables (lv 2)"
	echo -e "\t\t2\ttc (lv 3-4)"
	echo -e "\t\t3\tiptables (lv 3-4)"
	echo -e "\t\t4\tcilium (lv 3-4)"
	echo -e "\t\t5\tcilium (lv 7)"
	echo -e "\t\t6\tcilium (lv 3-4-7)\n"
	exit
elif [ "$1" = "start" ] && [ $# -lt 3 ]; then
	echo -e "Utilizzo:\t$0 start {file_puppet_init} {ID_FIREWALL}"
	echo -e "Per:\t\teseguire una rete di container e installare su essa un firewall\n"
	echo -e "\t{ID_FIREWALL}:"
	echo -e "\t\t 0\tnessun firewall"
	echo -e "\t\t 1\tebtables (lv 2)"
	echo -e "\t\t 2\ttc (lv 3-4)"
	echo -e "\t\t 3\tiptables (lv 3-4)"
	echo -e "\t\t 4\tcilium (lv 3-4)"
	echo -e "\t\t 5\tmodsecurity (lv 7)"
	echo -e "\t\t 6\tcilium (lv 7)"
	echo -e "\t\t 7\tebtables + modsecurity (lv 2-7)"
	echo -e "\t\t 8\ttc + modsecurity (lv 3-4-7)"
	echo -e "\t\t 9\tiptables + modsecurity (lv 3-4-7)"
	echo -e "\t\t10\tcilium + modsecurity (lv 3-4-7)"
	echo -e "\t\t11\tcilium (lv 3-4-7)\n"
	exit
elif [ "$1" = "stop" ] && [ $# -lt 3 ]; then
	echo -e "Utilizzo:\t$0 stop {file_puppet_exit} {LIVELLO_PULIZIA}"
	echo -e "Per:\t\tdistruggere una rete di container e il firewall installato su essa\n"
	echo -e "\t{LIVELLO_PULIZIA}:"
	echo -e "\t\t0\tesegui il primo livello di pulizia"
	echo -e "\t\t1\tesegui i livelli di pulizia 0 e 1"
	echo -e "\t\t2\tesegui tutti i livelli di pulizia\n"
	exit
elif [ "$1" = "uninstall" ] && [ $# -lt 2 ]; then
	echo -e "Utilizzo:\t$0 uninstall {ID_FIREWALL}"
	echo -e "Per:\t\tdisinstallare un firewall precedentemente installato su una rete di container\n"
	echo -e "\t{ID_FIREWALL}:"
	echo -e "\t\t1\tebtables (lv 2)"
	echo -e "\t\t2\ttc (lv 3-4)"
	echo -e "\t\t3\tiptables (lv 3-4)"
	echo -e "\t\t4\tcilium (lv 3-4|7|3-4-7)\n"
	exit
fi
working_dir="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
cd $working_dir


# TRANSLATE_FIREWALL_RULES #
function translate_firewall_rules {
	cp $1 $1.bkp

	# Traduci i Nomi dei Container nei Rispettivi Indirizzi MAC nel Firewall di Livello 2
	awk '/##### FIREWALL #####/,0' $1.bkp | sed -E "/firewall[ ]?\{/q" - | head -n -1 > $1
	names=( $(cat $1 | grep "[^_]source\|[^_]destination" | cut -d '"' -f 2 | cut -d "'" -f 2) )
	for name in "${names[@]}"; do
		if [[ $name != *"$"* ]]; then
			mac=$(docker inspect $name | grep -m 1 '"MacAddress"' | cut -d '"' -f 4)
			sed -i "s/$name/$mac/g" $1
		fi
	done

	# Traduci i Nomi dei Container nei Rispettivi Indirizzi IP nel Firewall di Livello 3-4
	awk '/firewall[ ]?{/,0' $1.bkp | sed '/##### CILIUM #####/q' - | head -n -1 >> $1
	names=( $(cat $1 | grep "[^_]source\|[^_]destination" | cut -d '"' -f 2 | cut -d "'" -f 2) )
	for name in "${names[@]}"; do
		if [[ $name != *"$"* ]]; then
			ip=$(docker inspect $name 2>/dev/null | grep -m 1 '"IPAddress"' | cut -d '"' -f 4)
			if [ "$ip" != "" ]; then
				sed -i "s/$name/$ip/g" $1
			fi
		fi
	done

}


# GENERATE_EBTABLES_FIREWALL #
function generate_ebtables_firewall {

	# Definisci le Regole di Traduzione
	declare -A filters
	filters=( ["proto"]="-p" ["source"]="-s" ["destination"]="-d" ["ip_source"]="--ip-source" ["ip_destination"]="--ip-destination"
			["ip_proto"]="--ip-protocol" ["ip_sport"]="--ip-source-port" ["ip_dport"]="--ip-destination-port" )

	# Genera le Varie Regole del Firewall e Scrivile nel File di Configurazione per Puppet
	echo -e "##### FIREWALL #####\n" > init.pp
	echo -e "\$system_path = \"$PATH\"\n" >> init.pp
	tail -n +2 $1 | sed -E "/firewall[ ]?\{/q" - | head -n -1 | while read line; do

		# Commenti, Righe Vuote e Variabili #
		if [ "$line" = "" ] || [[ ${line:0:1} == '#' ]] || [[ ${line:0:1} == '$' ]]; then
			echo $line >> init.pp

		# Intestazione Regole #
		elif [[ "$line" == firewall* ]]; then
			rule_name=$(echo $line | cut -d '"' -f 2 | cut -d "'" -f 2)
			echo -e "exec { '$rule_name':" >> init.pp
			cmd="\tcommand => \"ebtables -A |CHAIN|"
			rule_action=""

		# Chiusura Regole #
		elif [[ ${line:0:1} == '}' ]]; then
			if [ "$rule_action" = "LOG" ]; then
				cmd="$cmd --log --log-ip\""
			else
				cmd="$cmd -j $rule_action\""
			fi
			echo -e "${cmd},\n\tpath => \"\${system_path}\"\n}" >> init.pp
			if [[ "$cmd" == *"-s"* ]] || [[ "$cmd" == *"-d"* ]] || \
					[[ "$cmd" == *"--ip-source"* ]] || [[ "$cmd" == *"--ip-destination"* ]]; then
				cmd="${cmd/ -s / -old_s }"
				cmd="${cmd/--ip-source/--old_ip-source}"
				cmd="${cmd/--ip-source-port/--old_ip-source-port}"
				cmd="${cmd/ -d / -s }"
				cmd="${cmd/--ip-destination/--ip-source}"
				cmd="${cmd/--ip-destination-port/--ip-source-port}"
				cmd="${cmd/-old_s/-d}"
				cmd="${cmd/--old_ip-source/--ip-destination}"
				cmd="${cmd/--old_ip-source-port/--ip-destination-port}"
				echo -e "exec { '${rule_name}_opposite':" >> init.pp
				echo -e "${cmd},\n\tpath => \"\${system_path}\"\n}" >> init.pp
			fi

		# Parametri Regole #
		else
			key=$(echo ${line%%=>*})
			value=$(echo ${line##*=>} | cut -d '"' -f 2 | cut -d "'" -f 2)
			if [ "$key" = "action" ] || [ "$key" = "jump" ]; then
				rule_action=$(echo "$value" | tr a-z A-Z)
			elif [ "$key" = "chain" ]; then
				cmd="${cmd/|CHAIN|/$value}"
			elif [ "${filters[$key]}" != "" ]; then
				cmd="$cmd ${filters[$key]} $value"
			fi

		fi
	done

}


# GENERATE_TC_FIREWALL #
function generate_tc_firewall {

	# Definisci le Regole di Traduzione
	declare -A filters
	filters=( ["proto"]="match ip protocol |K| 0xff" ["proto_all"]="match ip protocol 0 0x00" \
			["source"]="match ip src |K|" ["destination"]="match ip dst |K|" \
			["sport"]="match ip sport |K| 0xffff" ["dport"]="match ip dport |K| 0xffff" ["action"]="action |K|" )
	declare -A filters_keys
	filters_keys=( ["icmp"]="1" ["tcp"]="6" ["udp"]="17" ["accept"]="pass" )

	# Aggiungi l'Inizializzazione di TC su Tutte le Interfacce Docker al File di Configurazione per Puppet
	echo -e "##### FIREWALL #####\n" > init.pp
	echo -e "\$system_path = \"$PATH\"\n" >> init.pp
	ifaces=( $(ip addr | grep veth | cut -d ' ' -f 2 | cut -d '@' -f 1) )
	echo -e "# Inizializza Tc sulle Interfacce di Ogni Container Creato #" >> init.pp
	for iface in "${ifaces[@]}"; do
		echo -e "exec { '${iface}_init':\n\tcommand => \"tc qdisc add dev $iface ingress\"," >> init.pp
		echo -e "\tpath => \"\${system_path}\"\n}" >> init.pp
	done
	echo "" >> init.pp

	# Genera le Varie Regole del Firewall e Scrivile nel File di Configurazione per Puppet
	rule_num=1
	manage_block=0
	sed '/firewall_l7/q' $1 | head -n -2 | while read line; do

		# Commenti, Righe Vuote e Variabili #
		if [ "$line" = "" ] || [[ ${line:0:1} == '#' ]] || [[ ${line:0:1} == '$' ]]; then
			echo $line >> init.pp

		# Intestazione Regole #
		elif [[ "$line" == firewall* ]]; then
			if [[ "$line" == firewall_l2* ]]; then
				manage_block=0
				continue
			fi
			manage_block=1
			action=""
			cmd_name=$(echo $line | cut -d "'" -f 2 | cut -d '"' -f 2)
			rule_id=${cmd_name%% *}
			rule_id=$(( 10#$rule_id ))
			rule_id=$(( rule_id + 1 ))
			cmd="tc filter add dev |I| protocol ip parent ffff: pref $rule_id u32"

		# Chiusura Regole #
		elif [[ ${line:0:1} == '}' ]]; then
			if [ $manage_block = "0" ]; then
				continue
			fi
			if [ "$action" != "" ]; then	# Aggiunta dell'Azione al Comando
				if [ "${filters_keys[$action]}" = "" ]; then
					cmd="$cmd action $action"
				else
					cmd="$cmd action ${filters_keys[$action]}"
				fi
			fi
			n_cycle=1
			while [ $n_cycle -lt 3 ]; do
				i=${cmd##*match ip src}	# Filtro su SRC corrente
				if [ "$i" != "$cmd" ]; then
					i=$(echo $i | cut -d ' ' -f 1)
					if [[ "$i" == "172.17.0."* ]]; then
						i=$(( ${i##*.} - 2 ))
						real_cmd="${cmd/|I|/${ifaces[$i]}}"
						echo -e "exec { '$cmd_name - tc_$rule_num':\n\tcommand => \"$real_cmd\"," >> init.pp
						echo -e "\tpath => \"\${system_path}\"\n}" >> init.pp
						(( rule_num++ ))
					fi
				fi
				i=${cmd##*match ip dst}	# Filtro su DST corrente
				if [ "$i" != "$cmd" ]; then
					i=$(echo $i | cut -d ' ' -f 1)
					if [[ "$i" == "172.17.0."* ]]; then
						i=$(( ${i##*.} - 2 ))
						real_cmd="${cmd/|I|/${ifaces[$i]}}"
						echo -e "exec { '$cmd_name - tc_$rule_num':\n\tcommand => \"$real_cmd\"," >> init.pp
						echo -e "\tpath => \"\${system_path}\"\n}" >> init.pp
						(( rule_num++ ))
					fi
				fi
				cmd="${cmd/match ip src/match ip |S|}"
				cmd="${cmd/match ip dst/match ip src}"
				cmd="${cmd/match ip |S|/match ip dst}"
				(( n_cycle++ ))
			done
			if [ "${cmd##*match ip src}" = "$cmd" ] && [ "${cmd##*match ip dst}" = "$cmd" ]; then
				for iface in ${ifaces[@]}; do	# Filtro su Tutti i Container
					real_cmd="${cmd/|I|/$iface}"
					echo -e "exec { '$cmd_name - tc_$rule_num':\n\tcommand => \"$real_cmd\"," >> init.pp
					echo -e "\tpath => \"\${system_path}\"\n}" >> init.pp
					(( rule_num++ ))
				done
			fi

		# Parametri Regole #
		else
			if [ $manage_block = "0" ]; then
				continue
			fi
			key=$(echo ${line%%=>*})
			value=$(echo ${line##*=>} | cut -d '"' -f 2 | cut -d "'" -f 2)
			if [ "$key" = "action" ]; then
				action="$value"
				continue
			elif [ "$key" = "jump" ] && [ "$value" = "LOG" ]; then
				manage_block=0
				continue
			elif [ "${filters[$key]}" = "" ]; then
				continue
			elif [ "$key" = "proto" ] && [ "$value" = "all" ]; then
				key="proto_all"
			fi
			cmd="$cmd ${filters[$key]}"
			if [ "${filters_keys[$value]}" = "" ]; then
				cmd="${cmd/|K|/$value}"
			else
				cmd="${cmd/|K|/${filters_keys[$value]}}"
			fi
		fi

	done

}


# EXECUTE_IN_CILIUM #
function execute_in_cilium {

	# Aggiungi la Creazione dell'Architettura Cilium al File di Configurazione per Puppet
	cp init.pp init.tmp
	sed '/##### CONTAINERS #####/q' init.tmp | head -n -1 > init.pp
	awk '/##### CILIUM #####/,0' $1 | sed '/##### CILIUM ROUTES #####/q' - | head -n -1 >> init.pp
	echo -e "\n\n" >> init.pp
	awk '/##### CONTAINERS #####/,0' init.tmp >> init.pp
	rm init.tmp

	# Aggiungi l'Esecuzione nella Rete Cilium al File di Configurazione per Puppet (se Necessario)
	net_name=$(cat init.pp | grep "docker network create")
	net_name=${net_name##* }
	net_name=${net_name%\"*}
	net_name=${net_name%\'*}
	cat init.pp | grep "docker::run" | while read line; do
		sed -i "s/$line/$line\n\tnet => \"$net_name\",/" init.pp
	done

	# Associa i Nomi dei Vari Container agli Indirizzi IP validi per la Rete Docker0
	names=( $(grep "name.*=>" init.pp | cut -d '=' -f 2 | cut -d "'" -f 2 | cut -d '"' -f 2) )
	ip_id=2
	declare -A ip_dict
	for name in "${names[@]}"; do
		if [[ "$name" == *\${*} ]]; then
			name_values=$(grep "$name.*=" init.pp)
			name_values="${name_values##*=}"
			if [ "$name_values" = "" ]; then
				name=$( echo $name | sed -e 's/{\(.*\)}/\1/')
				name_values=$(grep "$name.*=" init.pp)
				name_values="${name_values##*=}"
			fi
			name_values=( $(echo ${name_values//,/ }) )
			for i in "${!name_values[@]}"; do
				ip_dict+=(["172.17.0.$ip_id"]="${name_values[$i]:1:-1}")
				(( ip_id++ ))
			done
				(( ip_id-- ))
		else
			ip_dict+=(["172.17.0.$ip_id"]="$name")
		fi
		(( ip_id++ ))
	done

	# Aggiorna Tutti gli Indirizzi IP dei Container presenti nei Vari File nei Rispettivi Hostname (se Necessario)
	files=( $(grep -r --exclude=$(basename "$0") --exclude="*.bkp" "172.17.0.*" -m 1 . | cut -d ':' -f 1) )
	for file in "${files[@]}"; do
		cp $file $file.bkp
		for ip in "${!ip_dict[@]}"; do
			sed -i -e "s/$ip/${ip_dict[$ip]}/g" $file
		done
	done

	# Esegui la Rete di Container Senza Firewall
	puppet apply init.pp
	sleep 10

	# Aggiungi le Regole di Routing per Cilium nel File di Configurazione per Puppet (se Necessario)
	echo -e "# Inizializza Variabili #" > init.pp
	echo -e "\$pwd = \"$working_dir\"" >> init.pp
	echo -e "\$system_path = \"$PATH\"" >> init.pp
	awk '/##### CILIUM ROUTES #####/,0' $1 >> init.pp
	for name in "${names[@]}"; do
		if [[ "$name" == *\${*}* ]]; then
			name_values=$(grep "$name.*=" $1)
			name_values="${name_values##*=}"
			if [ "$name_values" = "" ]; then
				name=$( echo $name | sed -e 's/{\(.*\)}/\1/')
				name_values=$(grep "$name.*=" $1)
				name_values="${name_values##*=}"
			fi
			name_values=( $(echo ${name_values//,/ }) )
			for i in "${!name_values[@]}"; do
				container_ip=$(docker inspect "${name_values[$i]:1:-1}" | grep -E "\"IPAddress\": \"[0-9].*\"" | cut -d '"' -f 4)
				sed -i -e "s/$name/$container_ip/g" init.pp
			done
		else
			container_ip=$(docker inspect $name | grep -E "\"IPAddress\": \"[0-9].*\"" | cut -d '"' -f 4)
			sed -i -e "s/$name/$container_ip/g" init.pp
		fi
	done

}


# MOVE_TO_CILIUM #
function move_to_cilium {

	# Inizializza l'Architettura Cilium
	echo -e "\$pwd = \"$working_dir\"" > init.pp
	echo -e "\$system_path = \"$PATH\"\n" >> init.pp
	awk '/##### CILIUM #####/,0' $1 | sed '/##### CILIUM ROUTES #####/q' - | head -n -1 >> init.pp
	puppet apply init.pp
	sleep 10
	cat init.pp | grep "ROOT_PATH\|CONTAINERS" > cilium_uninstall_support.tmp

	# Inizializza le Variabili Utili
	containers=$(cat init.pp | grep CONTAINERS)
	containers=${containers#*CONTAINERS}
	containers=${containers#*:}
	IFS=', ' read -r -a containers <<< $containers
	declare -A old_ips
	for container in "${containers[@]}"; do
		old_ip=$(docker inspect $container | grep -E -m 1 "\"IPAddress\": \"[0-9].*\"" | cut -d '"' -f 4)
		old_ips+=(["$container"]="$old_ip")
	done
	root_path=$(cat init.pp | grep ROOT_PATH)
	root_path=${root_path#*ROOT_PATH}
	root_path=${root_path#*:}
	services=$(cat init.pp | grep SERVICES)
	services=${services#*SERVICES}
	services=${services#*:}
	IFS=', ' read -r -a services <<< $services
	net_name=$(cat init.pp | grep "docker network create")
	net_name=${net_name##* }
	net_name=${net_name%\"*}
	net_name=${net_name%\'*}

	# Aggiorna Tutti gli Indirizzi IP dei Container presenti nei Vari File nei Rispettivi Hostname (se Necessario)
	for container in "${containers[@]}"; do
		files=( $(grep -r --exclude=$(basename "$0") --exclude="*.bkp" "${old_ips[$container]}" -m 1 $root_path | cut -d ':' -f 1) )
		for file in "${files[@]}"; do
			cp -n $file $file.bkp
			sed -i -e "s/${old_ips[$container]}/$container/g" $file
		done
	done

	# Sposta i Container nella Rete Cilium
	docker network ls -q | while read net_id; do
		docker inspect $net_id | grep Name | while read line; do
			for container in "${containers[@]}"; do
				if [[ "$line" = *"$container"* ]]; then
					docker network disconnect $net_id $container
				fi
			done
		done
	done
	for container in "${containers[@]}"; do
		docker network connect $net_name $container
	done

	# Rigenera tutti i Container per l'Associazione al Daemon di Cilium
	for container in "${containers[@]}"; do
		is_service=0
		for service in "${services[@]}"; do
			if [ $container = $service ]; then
				is_service=1
				break
			fi
		done
		if [ $is_service = 0 ]; then
			docker restart $container
		fi
	done
	sleep 30
	for service in "${services[@]}"; do
		docker restart $service
	done

	# Aggiungi le Regole di Routing per Cilium nel File di Configurazione per Puppet (se Necessario)
        echo -e "# Inizializza Variabili #" > init.pp
        echo -e "\$pwd = \"$working_dir\"" >> init.pp
        echo -e "\$system_path = \"$PATH\"" >> init.pp
        awk '/##### CILIUM ROUTES #####/,0' $1 >> init.pp
        for container in "${containers[@]}"; do
                container_ip=$(docker inspect $container | grep -E "\"IPAddress\": \"[0-9].*\"" | cut -d '"' -f 4)
                sed -i -e "s/$container/$container_ip/g" init.pp
        done

}


# GENERATE_CILIUM_FIREWALL #
function generate_cilium_firewall {

	# Esegui la Rete di Container senza Firewall
	if [ "$1" != "11" ]; then
		if [ "$4" = "0" ]; then
			execute_in_cilium $2
		else
			move_to_cilium $2
		fi
	fi

	# Genera le Varie Regole del Firewall
	declare -A names_to_labels
	endpoints=( $(cilium endpoint list --no-headers | cut -d ' ' -f 1) )
	docker_ids=( $(docker ps -q) )
	for endpoint_id in "${endpoints[@]}"; do
		endpoint_ip=$(cilium endpoint get $endpoint_id | grep ipv4 | cut -d '"' -f 4)
		for docker_id in "${docker_ids[@]}"; do
			if [[ "$(docker inspect $docker_id | grep $endpoint_ip)" != "" ]]; then
				endpoint_name=$(docker inspect $docker_id | grep -m 1 Name | cut -d '"' -f 4 | cut -d '/' -f 2)
				break
			fi
		done
		endpoint_label=$(cilium endpoint get $endpoint_id | grep -m 1 "container:" | cut -d ':' -f 2)
		if [ "$endpoint_label" = "" ]; then
			continue
		fi
		names_to_labels+=(["$endpoint_name"]="${endpoint_label::-1}")
	done
	rule_num=$(ls $3 -1 | wc -l)
	manage_block=0
	declare -A vars
	sed -e '1,/##### FIREWALL #####/d' $2 | sed '/##### CILIUM #####/q' - | head -n -1 | while read line; do

		# Commenti e Righe Vuote #
		if [ "$line" = "" ] || [[ ${line:0:1} == '#' ]]; then
			continue

		# Variabili #
		elif [[ ${line:0:1} == '$' ]]; then
			if [ "$1" != "11" ]; then
				echo "$line" >> init.pp
			fi
			name=$(echo ${line%%=*})
			value=$(echo ${line##*=})
			value=$(echo ${value#*\"})
			value=$(echo ${value%\"*})
			vars+=(["\${${name##*$}}"]="$value")

		# Intestazione Regole #
		elif [[ "$line" == firewall* ]]; then
			if ([[ "$line" == firewall_l7* ]] && [ "$1" = "6" ]) || \
					([[ "$line" != firewall_l2* ]] && [[ "$line" != firewall_l7* ]] && [ "$1" != "6" ]); then
				manage_block=1
			else
				continue
			fi
			basic_rule_1="[{\n\t\"labels\": [{\"key\": \"name\", \"value\": \"|RULE_NAME|\"}],\n\t\"endpointSelector\": {\"matchLabels\": {\"|KEY_DST|\":\"|VALUE_DST|\"}},"
			basic_rule_2_1="\t\"ingress\": [{\n\t\t\"fromEndpoints\": [{\"matchLabels\": {\"|KEY_SRC|\":\"|VALUE_SRC|\"}}],"
			basic_rule_2_2="\t\"ingress\": [{\n\t\t\"fromEndpoints\": [{\"matchLabels\": {}}],"
			basic_rule_2_3="\t\"ingress\": [{\n\t\t\"fromCIDR\": [\"|IP|\"],"
			basic_rule_3_1="\t\t\"toPorts\": [{\"ports\": [{\"port\": \"|PORT|\", \"protocol\": \"|PROTO|\"}]}]"
			basic_rule_3_2="\t\t\"toPorts\": [{\"ports\": [{\"port\": \"|PORT|\"}]}]"
			basic_rule_3_3="\t\t\"toPorts\": [{\n\t\t\t\"ports\": [{\"port\": \"|PORT|\", \"protocol\": \"TCP\"}],\n\t\t\t\"rules\": {\"HTTP\": [{\"path\": \"|PATH|\"}]}\n\t\t}]"
			basic_rule_3_4="\t\t\"toPorts\": [{\n\t\t\t\"ports\": [{\"port\": \"|PORT|\", \"protocol\": \"TCP\"}],\n\t\t\t\"rules\": {\"HTTP\": [{\"headers\": [\"|HEADER_KEY|: |HEADER_VALUE|\"]}]}\n\t\t}]"
			basic_rule_4="\t}]\n}]"
			label_src=""
			label_dst=""
			rule_name=$(echo $line | cut -d "'" -f 2 | cut -d '"' -f 2)
			(( rule_num++ ))
			basic_rule_1="${basic_rule_1/|RULE_NAME|/$rule_name}"

		# Chiusura Regole #
		elif [[ ${line:0:1} == '}' ]]; then
			if [ $manage_block = "1" ] && [ "$1" = "11" ]; then
				source="\\\"${label_src%%=*}\\\":\\\"${label_src##*=}\\\""
				destination="\\\"${label_dst%%=*}\\\":\\\"${label_dst##*=}\\\""
				rules_l7=( $(grep "{\"path\":\|{\"headers\":" $3/* | cut -d ':' -f 1) )
				for rule in "${rules_l7[@]}"; do
					if [ "$(cat $rule | grep $source)" != "" ] && [ "$(cat $rule | grep $destination)" != "" ]; then
						(( rule_num-- ))
						manage_block=0	# Ignora Regola Lv3 se ne Esiste già una Equivalente di Lv7
						break
					fi
				done
			fi
			if [ $manage_block = "0" ]; then
				continue
			else
				manage_block=0
			fi
			if [[ "$basic_rule_1" != *"|"*"|"* ]]; then
				echo -e ${basic_rule_1} > "$3/rule_$rule_num"
			else
				(( rule_num-- ))
				continue
			fi
			if [[ "$basic_rule_2_1" != *"|"*"|"* ]]; then
				basic_rule_2=${basic_rule_2_1}
			elif [[ "$basic_rule_2_3" != *"|"*"|"* ]]; then
				basic_rule_2=${basic_rule_2_3}
			else
				basic_rule_2=${basic_rule_2_2}
			fi
			if [[ "$basic_rule_3_3" != *"|PATH|"* ]]; then
				echo -e ${basic_rule_2} >> "$3/rule_$rule_num"
				echo -e ${basic_rule_3_3} >> "$3/rule_$rule_num"
			elif [[ "$basic_rule_3_4" != *"|HEADER_KEY|"* ]]; then
				echo -e ${basic_rule_2} >> "$3/rule_$rule_num"
				echo -e ${basic_rule_3_4} >> "$3/rule_$rule_num"
			elif [[ "$basic_rule_3_1" == *"|PORT|"* ]]; then
				echo -e "${basic_rule_2::-1}" >> "$3/rule_$rule_num"
			elif [[ "$basic_rule_3_1" == *"|PROTO|"* ]]; then
				echo -e ${basic_rule_2} >> "$3/rule_$rule_num"
				echo -e ${basic_rule_3_2} >> "$3/rule_$rule_num"
			else
				echo -e ${basic_rule_2} >> "$3/rule_$rule_num"
				echo -e ${basic_rule_3_1} >> "$3/rule_$rule_num"
			fi
			echo -e ${basic_rule_4} >> "$3/rule_$rule_num"

		# Parametri Regole #
		else
			if [ $manage_block = "0" ]; then
				continue
			fi
			key=$(echo ${line%%=>*})
			if [ "$key" = "rule" ]; then
				value=$(echo ${line##*=>})
				value=$(echo ${value#*\"})
				value=$(echo ${value%\"*})
			else
				value=$(echo ${line##*=>} | cut -d '"' -f 2 | cut -d "'" -f 2 | cut -d ',' -f 1)
			fi
			if ([ "$key" = "action" ] && [ "$value" = "drop" ]) || ([ "$key" = "jump" ] && [ "$value" = "LOG" ]); then # Azione
				manage_block=0
				continue
			elif [ "$key" = "proto" ]; then	# Protocollo (lv 3-4)
				basic_rule_3_1="${basic_rule_3_1/|PROTO|/$value}"
			elif [ "$key" = "dport" ]; then	# Porta Destinazione (lv 3-4)
				basic_rule_3_1="${basic_rule_3_1/|PORT|/$value}"
				basic_rule_3_2="${basic_rule_3_2/|PORT|/$value}"
			elif [[ "$key" == *source ]]; then	# Sorgente (lv 3-4-7)
				label_src="${names_to_labels[$value]}"
				if [ "$label_src" = "" ]; then
					if [[ ${vars["$value"]} != "" ]]; then
						basic_rule_2_3="${basic_rule_2_3/|IP|/${vars[$value]}}"
					else
						basic_rule_2_3="${basic_rule_2_3/|IP|/$value}"
					fi
				else
					basic_rule_2_1="${basic_rule_2_1/|KEY_SRC|/${label_src%%=*}}"
					basic_rule_2_1="${basic_rule_2_1/|VALUE_SRC|/${label_src##*=}}"
				fi
			elif [[ "$key" == *destination ]]; then	# Destinazione (lv 3-4-7)
				port=${value##*:}
				value=${value%%:*}
				label_dst="${names_to_labels[$value]}"
				basic_rule_1="${basic_rule_1/|KEY_DST|/${label_dst%%=*}}"
				basic_rule_1="${basic_rule_1/|VALUE_DST|/${label_dst##*=}}"
				if [ "$port" != "" ]; then
					basic_rule_3_3="${basic_rule_3_3/|PORT|/$port}"
					basic_rule_3_4="${basic_rule_3_4/|PORT|/$port}"
				fi
			elif [ "$key" = "rule" ]; then	# Regola Livello 7 (lv 7)
				rule_type=$(echo $value | cut -d ' ' -f 2)
				if [ "$rule_type" = "REQUEST_URI" ]; then
					uri=$(echo $value | cut -d ' ' -f 4)
					uri=${uri%\"*}
					uri=$(echo ${uri//\\/\\\\\\\\})
					basic_rule_3_3="${basic_rule_3_3/|PATH|/$uri}"
				elif [[ "$rule_type" == REQUEST_HEADERS* ]]; then
					header_key=${rule_type##*:}
					basic_rule_3_4="${basic_rule_3_4/|HEADER_KEY|/$header_key}"
					header_value=$(echo $value | cut -d ' ' -f 3)
					header_value=$(echo ${header_value#*\"})
					header_value=$(echo ${header_value%\"*})
					if [[ ${vars["$header_value"]} != "" ]]; then
						header_value=${vars["$header_value"]}
					fi
					basic_rule_3_4="${basic_rule_3_4/|HEADER_VALUE|/$header_value}"
				fi
			fi
		fi

	done

	# Elimina le Regole del Firewall Ridondanti
	for current_rule in $3/*; do
		for other_rule in $3/*; do
			if [ "$current_rule" != "$other_rule" ] && [ -f $current_rule ] && [ -f $other_rule ] && \
								[ "$(diff -I labels $current_rule $other_rule)" = "" ]; then
				rm $other_rule
			fi
		done
	done

}


# GENERATE_MODSECURITY_FIREWALL #
function generate_modsecurity_firewall {

	# Scrivi le Regole del Firewall Modsecurity nei Rispettivi File di Configurazione
	target="/dev/null"
	realpath $1 > modsecurity_stop_support.tmp
	sed -e '1,/firewall_l7/d' $1 | sed '/##### CILIUM #####/q' - | head -n -1 | while read line; do
		key=$(echo ${line%%=>*})
		value=$(echo ${line##*=>})
		value=$(echo ${value#*\"})
		value=$(echo ${value%\"*})
		if [ "$key" = "modsecurity_target" ]; then
			target="${value/\$\{pwd\}\//}"
		elif [ "$key" = "rule" ]; then
			if [[ "$value" == *"\${"*"}"* ]]; then
				var_name=$(echo ${value#*\$\{})
				var_name=$(echo ${var_name%\}*})
				var_value=$(cat $1 | grep "\$$var_name" | cut -d '=' -f 2)
				var_value=$(echo ${var_value#*\"})
				var_value=$(echo ${var_value%\"*})
				value="${value/\$\{$var_name\}/$var_value}"
			fi
			echo $value >> $target
		elif [[ "${line:0:1}" == '}' ]]; then
			target="/dev/null"
		fi
	done
}


# START_LOGGER #
function start_logger {
	mkdir logs 2>/dev/null

	# Avvia il Log del Firewall Iptables/Ebtables (+ Modsecurity) (se Necessario) #
	if [ "$1" = "1" ] || [ "$1" = "3" ] || [ "$1" = "7" ] || [ "$1" = "9" ]; then
		if [[ $(grep ".*jump.*=>.*LOG.*" $2) != "" ]]; then
			cat /proc/kmsg | stdbuf -oL grep "IN=.*OUT=.*" > logs/firewall.log &
			pid=$!
			(( pid-- ))
			echo $pid > logger_kill_support.tmp
		fi

	# Avvia il Log del Firewall Cilium (+ Modsecurity) (se Necessario) #
	else
		log_types=$(cat $2 | grep "EVENT_TYPE")
		log_types=${log_types#*EVENT_TYPE}
		log_types=${log_types#*:}
		if [[ "$log_types" = *"none"* ]]; then
			return
		fi
		IFS=', ' read -r -a log_types <<< $log_types
		verbose_log=$(cat $2 | grep "VERBOSE_LOG")
		verbose_log=${verbose_log#*VERBOSE_LOG}
		verbose_log=${verbose_log#*:}
		if [[ "$verbose_log" = *"0"* ]]; then
			verbose_log=""
		else
			verbose_log="-v"
		fi
		for log_type in "${log_types[@]}"; do
			cilium monitor -t $log_type $verbose_log > logs/firewall_${log_type}.log &
			pid=$!
			echo $pid >> logger_kill_support.tmp
		done
	fi

}


# Exec #
if [ "$1" = "exec" ]; then

	# Inizializza il File di Configurazione per Puppet
	echo -e "# Inizializza Variabili #" > exec.pp
	echo -e "\$pwd = \"$working_dir\"" >> exec.pp
	echo -e "\$system_path = \"$PATH\"\n" >> exec.pp
	cat $2 >> exec.pp

	# Esegui Puppet
	puppet apply exec.pp

	# Rimuovi il File di Configurazione Generato
	rm exec.pp


# Install #
elif [ "$1" = "install" ]; then

	# Inizializza il File di Configurazione per Puppet
	echo -e "# Inizializza Variabili #" > init.pp
	echo -e "\$pwd = \"$working_dir\"" >> init.pp
	echo -e "\$system_path = \"$PATH\"\n" >> init.pp

	# Genera le regole del Firewall Ebtables e Scrivile nel File di Configurazione per Puppet (se Necessario)
	if [ "$3" = "1" ]; then
		ebtables-save > ebtables_preinstall.bkp
		translate_firewall_rules $2
		generate_ebtables_firewall $2
		start_logger $3 $2

	# Genera le regole del Firewall TC e Scrivile nel File di Configurazione per Puppet (se Necessario)
	elif [ "$3" = "2" ]; then
		translate_firewall_rules $2
		generate_tc_firewall $2

	# Copia le regole del Firewall Iptables nel File di Configurazione per Puppet (se Necessario)
	elif [ "$3" = "3" ]; then
		iptables-save > iptables_preinstall.bkp
		translate_firewall_rules $2
		sed '/firewall_l7/q' $2 | head -n -2 > tmp.pp
		cat tmp.pp | grep '\$.*=.*' > init.pp
		awk '/firewall[ ]?{/,0' tmp.pp | head -n -2 >> init.pp
		rm tmp.pp
		start_logger $3 init.pp

	# Genera le regole del Firewall Cilium e Scrivile nel File di Configurazione per Puppet (se Necessario)
	elif [ "$3" = "4" ] || [ "$3" = "5" ] || [ "$3" = "6" ]; then
		iptables-save > iptables_preinstall.bkp
		mkdir cilium 2>/dev/null
		filters_path="cilium/cilium_filters"
		mkdir $filters_path
		if [ "$3" = "4" ]; then
			generate_cilium_firewall 4 $2 $filters_path 1
		elif [ "$3" = "5" ]; then
			generate_cilium_firewall 6 $2 $filters_path 1
		else
			generate_cilium_firewall 6 $2 $filters_path 1
			generate_cilium_firewall 11 $2 $filters_path 1
		fi
		echo -e "\n\n\n##### FIREWALL #####\n" >> init.pp
		for rule_name in $filters_path/*; do
			echo -e "exec { '$(basename $rule_name)_import':\n\tcommand => \"cilium policy import \${pwd}/$rule_name\"," >> init.pp
			echo -e "\tpath => \"\${system_path}\"\n}" >> init.pp
		done
		start_logger $3 $2

	# Errore
	else
		echo -e "\n[E] Parametri di input errati!\n"
		rm init.pp
		exit
	fi

	# Esegui Puppet
	puppet apply init.pp

	# Rimuovi il File di Configurazione Generato
	rm init.pp
	rm -r cilium/cilium_filters 2>/dev/null


# Start #
elif [ "$1" = "start" ]; then

	# Crea un BackUp delle Regole di Filtraggio/Routing Presenti in Iptables/Ebtables
	iptables-save > iptables_prestart.bkp
	ebtables-save > ebtables_prestart.bkp

	# Inizializza il File di Configurazione per Puppet
	echo -e "# Inizializza Variabili #" > init.pp
	echo -e "\$pwd = \"$working_dir\"" >> init.pp
	echo -e "\$system_path = \"$PATH\"" >> init.pp
	sed -E '/(##### FIREWALL #####|##### CILIUM)/q' $2 | head -n -1 >> init.pp

	# Genera le regole del Firewall Ebtables (+ Modsecurity) e Scrivile nel File di Configurazione per Puppet (se Necessario)
	if [ "$3" = "1" ] || [ "$3" = "7" ]; then
		if [ "$3" = "7" ]; then
			generate_modsecurity_firewall $2
		fi
		puppet apply init.pp
		sleep 10
		translate_firewall_rules $2
		generate_ebtables_firewall $2
		start_logger $3 $2

	# Genera le regole del Firewall TC (+ Modsecurity) e Scrivile nel File di Configurazione per Puppet (se Necessario)
	elif [ "$3" = "2" ] || [ "$3" = "8" ]; then
		if [ "$3" = "8" ]; then
			generate_modsecurity_firewall $2
		fi
		puppet apply init.pp
		sleep 10
		translate_firewall_rules $2
		generate_tc_firewall $2

	# Copia le regole del Firewall Iptables (+ Modsecurity) nel File di Configurazione per Puppet (se Necessario)
	elif [ "$3" = "3" ] || [ "$3" = "9" ]; then
		if [ "$3" = "9" ]; then
			generate_modsecurity_firewall $2
		fi
		puppet apply init.pp
		sleep 10
		translate_firewall_rules $2
		sed '/firewall_l7/q' $2 | head -n -2 > tmp.pp
		cat tmp.pp | grep '\$.*=.*' > init.pp
		awk '/firewall[ ]?{/,0' tmp.pp | head -n -2 >> init.pp
		rm tmp.pp
		start_logger $3 init.pp

	# Genera le regole del Firewall Cilium (+ Modsecurity) e Scrivile nel File di Configurazione per Puppet (se Necessario)
	elif [ "$3" = "4" ] || [ "$3" = "6" ] || [ "$3" = "10" ] || [ "$3" = "11" ]; then
		mkdir cilium 2>/dev/null
		filters_path="cilium/cilium_filters"
		mkdir $filters_path
		if [ "$3" = "10" ]; then
			generate_modsecurity_firewall $2
			generate_cilium_firewall 4 $2 $filters_path 0
		elif [ "$3" != "11" ]; then
			generate_cilium_firewall $3 $2 $filters_path 0
		else
			generate_cilium_firewall 6 $2 $filters_path 0
			generate_cilium_firewall $3 $2 $filters_path 0
		fi
		echo -e "\n\n\n##### FIREWALL #####\n" >> init.pp
		for rule_name in $filters_path/*; do
			echo -e "exec { '$(basename $rule_name)_import':\n\tcommand => \"cilium policy import \${pwd}/$rule_name\"," >> init.pp
			echo -e "\tpath => \"\${system_path}\"\n}" >> init.pp
		done
		start_logger $3 $2

	# Genera le regole del Firewall Modsecurity e Scrivile nei Rispettivi File di Configurazione (se Necessario)
	elif [ "$3" = "5" ]; then
		generate_modsecurity_firewall $2

	# Errore
	elif [ "$3" != "0" ]; then
		echo -e "\n[E] Parametri di input errati!\n"
		rm init.pp iptables_prestart.bkp ebtables_prestart.bkp
		exit
	fi

	# Esegui Puppet
	puppet apply init.pp

	# Rimuovi il File di Configurazione Generato (& i Filtri Cilium)
	rm init.pp
	rm -r cilium/cilium_filters 2>/dev/null


# Stop #
elif [ "$1" = "stop" ]; then

	# Inizializza il File di Configurazione per Puppet
	echo -e "# Inizializza Variabili #" > exit.pp
	echo -e "\$pwd = \"$working_dir\"" >> exit.pp
	echo -e "\$system_path = \"$PATH\"\n" >> exit.pp

	# Genera il File di Configurazione per Puppet
	if [ "$3" = "0" ]; then
		sed '/##### LIVELLO_PULIZIA_1 #####/q' $2 | head -n -1 >> exit.pp
	elif [ "$3" = "1" ]; then
		sed '/##### LIVELLO_PULIZIA_2 #####/q' $2 | head -n -1 >> exit.pp
	elif [ "$3" = "2" ]; then
		cat $2 >> exit.pp
	else
		echo -e "\n[E] Parametri di input errati!\n"
	fi

	# Ferma il Log del Firewall (se Presente) #
	pids=( $(cat logger_kill_support.tmp 2>/dev/null) )
	kill -9 ${pids[@]} 2>/dev/null
	rm logger_kill_support.tmp 2>/dev/null

	# Esegui Puppet
	puppet apply exit.pp

	# Rimuovi il File di Configurazione Generato
	rm exit.pp

	# Ripristina le Regole di Filtraggio/Routing Originali di Iptables/Ebtables
	iptables-restore < iptables_prestart.bkp
	ebtables-restore < ebtables_prestart.bkp
	rm iptables_prestart.bkp ebtables_prestart.bkp

	# Ripristina i file Originali con quelli di Backup (se Necessario)
	files=( $(find . -name "*.bkp") )
	for file in "${files[@]}"; do
		if [[ "$file" != "./iptables"* ]] && [[ "$file" != "./ebtables"* ]]; then
			mv $file "${file::-4}"
		fi
	done

	# Ripulisci le Regole di Livello 7 create per Modsecurity
	grep "modsecurity_target" "$(cat modsecurity_stop_support.tmp 2>/dev/null)" 2>/dev/null | while read target; do
		target=$(echo $target | cut -d '"' -f 2 | cut -d "'" -f 2)
		target="${target/\$\{pwd\}\//}"
		echo "" > $target
	done
	rm modsecurity_stop_support.tmp 2>/dev/null


# Uninstall #
elif [ "$1" = "uninstall" ]; then
	root_path="."

	# Ripulisci le regole del Firewall Ebtables (se Necessario)
	if [ "$2" = "1" ]; then
		error=$(ebtables-restore 2>&1 < ebtables_preinstall.bkp)
		if [ "$?" = "0" ]; then
			echo "[ebtables-restore]: executed successfully"
		else
			echo "[ebtables-restore]: $error"
		fi
		rm ebtables_preinstall.bkp

	# Ripulisci le regole del Firewall TC (se Necessario)
	elif [ "$2" = "2" ]; then
		ifaces=( $(ip addr | grep veth | cut -d ' ' -f 2 | cut -d '@' -f 1) )
		for iface in "${ifaces[@]}"; do
			error=$(tc qdisc del dev $iface ingress 2>&1)
			if [ "$error" = "" ]; then
				echo "[tc del dev $iface]: executed successfully"
			else
				echo "[tc del dev $iface]: $error"
			fi
		done

	# Ripulisci le regole del Firewall Iptables (se Necessario)
	elif [ "$2" = "3" ] || [ "$2" = "4" ]; then
		error=$(iptables-restore 2>&1 < iptables_preinstall.bkp)
		if [ "$?" = "0" ]; then
			echo "[iptables-restore]: executed successfully"
		else
			echo "[iptables-restore]: $error"
		fi
		rm iptables_preinstall.bkp

	# Prepara la Pulizia delle regole del Firewall Cilium (se Necessario)
	elif [ "$2" = "4" ]; then
		root_path=$(cat cilium_uninstall_support.tmp | grep ROOT_PATH)
		root_path=${root_path#*ROOT_PATH}
		root_path=${root_path#*:}

	# Errore
	else
		echo -e "\n[E] Parametri di input errati!\n"
		exit
	fi

	# Ripristina i file Originali con quelli di Backup (se Necessario)
	files=( $(find $root_path -name "*.bkp") )
	for file in "${files[@]}"; do
		if [[ "$file" != "./iptables"* ]] && [[ "$file" != "./ebtables"* ]]; then
			mv $file "${file::-4}"
		fi
	done

	# Ferma il Log del Firewall (se Presente) #
	pids=( $(cat logger_kill_support.tmp 2>/dev/null) )
	kill -9 ${pids[@]} 2>/dev/null
	rm logger_kill_support.tmp 2>/dev/null

	# Ripulisci le regole del Firewall Cilium (se Necessario)
	if [ "$2" = "4" ]; then
		cilium policy delete --all > /dev/null
		containers=$(cat cilium_uninstall_support.tmp | grep CONTAINERS)
		containers=${containers#*CONTAINERS}
		containers=${containers#*:}
		IFS=', ' read -r -a containers <<< $containers
		for container in "${containers[@]}"; do
			systemctl restart docker-$container
		done
		docker stop cilium 2>/dev/null
		docker rm cilium 2>/dev/null
		docker stop cilium-docker-plugin 2>/dev/null
		docker rm cilium-docker-plugin 2>/dev/null
		docker stop cilium-kvstore 2>/dev/null
		docker rm cilium-kvstore 2>/dev/null
		rm cilium_uninstall_support.tmp
	fi


# Errore #
else
	echo -e "\n[E] Parametri di input errati!\n"
fi
