##### CONTAINERS ##### (Non cambiare/cancellare questa riga!)

# Costruisci l'Immagine Docker del Server (se Necessario) #
docker::image { 'web_server':
	docker_dir => "web_server"
}

# Esegui il Server in Diversi Container #
["web_server", "web_server", "web_server"].each |Integer $index, String $value| {
	docker::run { "${value}${index}":
		image => "${value}",
		volumes => ["${pwd}/web_server/source:/var/www/html", "${pwd}/web_server/waf_rules:/usr/share/modsecurity-crs/rules", "${pwd}/logs/${value}${index}:/var/log/apache2"],
		name => ["web-server${index}"],	# web-server${index} = "web-server0", "web-server1", "web-server2"
		labels => ["id=${value}"],
		extra_parameters => ["--rm"],
		before => Exec["${value}${index}_wait"]
	}
	exec { "${value}${index}_wait":
		command => "sleep 3",
		path => "${system_path}"
	}
}

# Costruisci l'Immagine Docker del Database (se Necessario) #
docker::image { 'web_server_db':
	docker_dir => "database"
}

# Esegui il Database #
docker::run { 'web_server_db':
	image => "web_server_db",
	env => ["MYSQL_ROOT_PASSWORD=root_pass"],
	name => ["web-server-db"],
	labels => ["id=web_server_db"],
	extra_parameters => ["--rm"],
	before => Exec["web_server_db_wait"]
}
exec { 'web_server_db_wait':
	command => "sleep 10",
	path => "${system_path}"
}

# Costruisci l'Immagine Docker del Servizio di Amministrazione (se Necessario) #
docker::image { 'web_server_admin':
	docker_dir => "admin"
}

# Esegui il Servizio di Amministrazione #
docker::run { 'web_server_admin':
	image => "web_server_admin",
	volumes => ["${pwd}/web_server/source:/containers_network_data/web_server_source", "${pwd}/web_server/waf_rules:/containers_network_data/web_server_modsec_rules", "${pwd}/load_balancer:/containers_network_data/haproxy_conf", "${pwd}/logs:/containers_network_data/logs"],
	name => ["web-server-admin"],
	labels => ["id=web_server_admin"],
	extra_parameters => ["-p 22:22", "--rm"],
	before => Exec["web_server_admin_wait"]
}
exec { 'web_server_admin_wait':
	command => "sleep 3",
	path => "${system_path}"
}

# Esegui il Load Balancer #
docker::run { 'load_balancer':
	image => "haproxy",
	volumes => ["${pwd}/load_balancer:/usr/local/etc/haproxy"],
	name => ["load-balancer"],
	labels => ["id=load_balancer"],
	extra_parameters => ["-p 80:80", "--rm"],
}



##### FIREWALL ##### (Non cambiare/cancellare questa riga!)

$chosen_ip = "192.168.1.10"

### Firewall Livello 2 ###
# Whitelist del Load Balancer #
firewall_l2 { '000 - load_balancer<->web_server0':
	source => "load-balancer",
	destination => "web-server0",
	action => "accept",
	chain => "FORWARD"
}
firewall_l2 { '001 - load_balancer<->web_server1':
	source => "load-balancer",
	destination => "web-server1",
	action => "accept",
	chain => "FORWARD"
}
firewall_l2 { '002 - load_balancer<->web_server2':
	source => "load-balancer",
	destination => "web-server2",
	action => "accept",
	chain => "FORWARD"
}
firewall_l2 { '003 - world<->load_balancer':
	destination => "load-balancer",
	action => "accept",
	chain => "INPUT"
}

# Whitelist dei Web Server #
firewall_l2 { '004 - web_server0<->database':
	source => "web-server0",
	destination => "web-server-db",
	action => "accept",
	chain => "FORWARD"
}
firewall_l2 { '005 - web_server1<->database':
	source => "web-server1",
	destination => "web-server-db",
	action => "accept",
	chain => "FORWARD"
}
firewall_l2 { '006 - web_server2<->database':
	source => "web-server2",
	destination => "web-server-db",
	action => "accept",
	chain => "FORWARD"
}

# Whitelist del Pannello di Amministrazione #
firewall_l2 { '007 - admin<->database':
	source => "web-server-admin",
	destination => "web-server-db",
	action => "accept",
	chain => "FORWARD"
}
firewall_l2 { '008 - containers<->admin LOG':
	destination => "web-server-admin",
	jump => "LOG",
	chain => "FORWARD"
}
firewall_l2 { '009 - world<->admin LOG':
	destination => "web-server-admin",
	jump => "LOG",
	chain => "INPUT"
}
firewall_l2 { '010 - chosen_ip<->admin':
	proto => "IPv4",
	ip_source => "${chosen_ip}",
	destination => "web-server-admin",
	action => "accept",
	chain => "INPUT"
}

# Elimina Tutti gli Altri Pacchetti (sulla Rete Docker) Tranne quelli del Protocollo ARP #
firewall_l2 { '011 - accept_all_arp_containers':
	proto => "arp",
	action => "accept",
	chain => "FORWARD"
}
firewall_l2 { '012 - accept_all_arp_world':
	proto => "arp",
	action => "accept",
	chain => "INPUT"
}
firewall_l2 { '013 - drop_all_containers':
	action => "drop",
	chain => "FORWARD"
}
firewall_l2 { '014 - drop_all_world':
	action => "drop",
	chain => "INPUT"
}



### Firewall Livello 3-4 ###
# Inserire qui sotto le informazioni necessarie all'attivazione del log nel caso di firewall cilium. #
# Nello specifico inserire la lista dei tipi di evento da salvare (o 'none' nel caso in cui non si volesse attivare il log) #
# & definire se si vuole un log basilare o più dettagliato. #
# (Non cambiare/cancellare le parole scritte in maiuscolo!) #
# EVENT_TYPES: capture, drop, l7
# VERBOSE_LOG: 0

# Whitelist del Load Balancer #
firewall { '000 - load_balancer<->web_server0':
	proto => "tcp",
	source => "load-balancer",
	destination => "web-server0",
	action => "accept",
	chain => "DOCKER"
}
firewall { '001 - load_balancer<->web_server1':
	proto => "tcp",
	source => "load-balancer",
	destination => "web-server1",
	action => "accept",
	chain => "DOCKER"
}
firewall { '002 - load_balancer<->web_server2':
	proto => "tcp",
	source => "load-balancer",
	destination => "web-server2",
	action => "accept",
	chain => "DOCKER"
}
firewall { '003 - world<->load_balancer':
	proto => "tcp",
	destination => "load-balancer",
	action => "accept",
	chain => "DOCKER"
}

# Whitelist dei Web Server #
firewall { '004 - web_server0<->database':
	proto => "tcp",
	source => "web-server0",
	destination => "web-server-db",
	action => "accept",
	chain => "DOCKER"
}
firewall { '005 - web_server1<->database':
	proto => "tcp",
	source => "web-server1",
	destination => "web-server-db",
	action => "accept",
	chain => "DOCKER"
}
firewall { '006 - web_server2<->database':
	proto => "tcp",
	source => "web-server2",
	destination => "web-server-db",
	action => "accept",
	chain => "DOCKER"
}

# Whitelist del Pannello di Amministrazione #
firewall { '007 - admin<->database':
	proto => "tcp",
	source => "web-server-admin",
	destination => "web-server-db",
	action => "accept",
	chain => "DOCKER"
}
firewall { '008 - all<->admin LOG':
	proto => "tcp",
	destination => "web-server-admin",
	jump => "LOG",
	chain => "DOCKER"
}
firewall { '009 - chosen_ip<->admin':
	proto => "tcp",
	source => "${chosen_ip}",
	destination => "web-server-admin",
	action => "accept",
	chain => "DOCKER"
}

# Elimina Tutti gli Altri Pacchetti (sulla Rete Docker) #
firewall { '010 - drop_all':
	proto => "all",
	action => "drop",
	chain => "DOCKER"
}



### Firewall Livello 7 ###
firewall_l7 { 'admin_rule':
	modsecurity_target => "${pwd}/web_server/waf_rules/rules.conf",
	cilium_source => "load-balancer",
	cilium_destination => "web-server0:80",
	rule => "SecRule REQUEST_HEADERS:x-forwarded-for \"${chosen_ip}\" \"id:100,phase:1,t:none,nolog,pass,ctl:ruleEngine=off\""
}
firewall_l7 { 'public_pages':
	modsecurity_target => "${pwd}/web_server/waf_rules/rules.conf",
	rule => "SecRule REQUEST_URI \"@rx ^/$|^/(?!private)[a-z]*\\.php(\\?id=)?[0-9]*$\" \"id:101,allow\""
}
firewall_l7 { 'public_pages':
	cilium_source => "load-balancer",
	cilium_destination => "web-server0:80",
	rule => "SecRule REQUEST_URI \"@rx ^/$|^/index.php$|^/public.php$|^/vulnerable.php(\\?id=)?[0-9]*$\" \"id:102,allow\""
}
firewall_l7 { 'deny_all':
	modsecurity_target => "${pwd}/web_server/waf_rules/rules.conf",
	rule => "SecRule REQUEST_URI \".\" \"id:103,severity:ERROR,deny,status:404\""
}



##### CILIUM ##### (Non cambiare/cancellare questa riga!)

# Inserire qui sotto le informazioni necessarie all'installazione del firewall cilium in una rete di container precedentemente creata. #
# Nello specifico inserire la lista dei container da spostare nella rete cilium separati da virgola #
# & il path (assoluto o relativo alla cartella 'Programmi/rete_container/') alla cartella root della rete di container #
# & la lista dei servizi da riavviare separati da virgola. #
# (Non cambiare/cancellare le parole scritte in maiuscolo!) #
# CONTAINERS: web-server0, web-server1, web-server2, web-server-db, web-server-admin, load-balancer
# ROOT_PATH: .
# SERVICES: load-balancer

# Installa Docker Compose #
class { 'docker::compose':
	version => "1.21.2"
}

# Costruisci e Esegui l'Architettura di Cilium #
docker_compose { "${pwd}/cilium/docker-compose.yml":
	ensure => present,
	before => Exec["cilium-net"]
}

# Crea la Rete Privata su cui i Container Comunicheranno #
exec { 'cilium-net':
	command => "docker network create --ipv6 --subnet ::1/112 --driver cilium --ipam-driver cilium cilium-net",
	path => "${system_path}",
	tries => 5,
	try_sleep => 3
}



##### CILIUM ROUTES ##### (Non cambiare/cancellare questa riga!)

# Regole di Routing per il Load Balancer #
firewall { '001 - world<->load_balancer':
	table => "nat",
	chain => "PREROUTING",
	dport => 80,
	jump => "DNAT",
	todest => "load-balancer:80"
}
firewall { '002 - localhost<->load_balancer':
	table => "nat",
	chain => "OUTPUT",
	outiface => "lo",
	dport => 80,
	jump => "DNAT",
	todest => "load-balancer:80"
}

# Regole di Routing per il Pannello di Amministrazione #
firewall { '003 - world<->web_server_admin':
	table => "nat",
	chain => "PREROUTING",
	dport => 22,
	jump => "DNAT",
	todest => "web-server-admin:22"
}
firewall { '004 - localhost<->web_server_admin':
	table => "nat",
	chain => "OUTPUT",
	outiface => "lo",
	dport => 22,
	jump => "DNAT",
	todest => "web-server-admin:22"
}

# Rimozione Regola di SNAT Creata Automaticamente da Cilium #
exec { 'cilium_snat':
	command => "iptables -D CILIUM_POST 1 -t nat",
	path => "${system_path}"
}
